@@ -54,7 +54,7 @@ class ExcelExporter:
         self._log = log
         self._path_to_excel = path_to_excel
 
-    def export_tasks(self, tasks_dict, date_offset):
+    def export_tasks(self, tasks_dict, date_offset, granularity):
         self._log.info("Starting export of tasks to '{}' ...".format(self._path_to_excel))
         success = version_1.tasks_to_excel(
             self._log,
@@ -64,6 +64,7 @@ class ExcelExporter:
             sheet_meta = excelcolumns.DEFAULT_SHEET_META,
             version = version,
             date_offset = date_offset,
+            horizon_granularity = granularity,
         )
         if success:
             self._log.info("Finished export to '{}'.".format(self._path_to_excel))
