@@ -6,7 +6,7 @@ class Task:
 
     DEFAULT_EFFORT = 1
 
-    def __init__(self, id = uuid.uuid4(), name = coolname.generate_slug(2), effort = 1, progress = 0.0, start = None, end = None,):
+    def __init__(self, id = uuid.uuid4(), name = coolname.generate_slug(2), effort = 1, progress = 0.0, start = None, end = None):
         self._id = id
         self._name = name
 
@@ -19,7 +19,6 @@ class Task:
 
         self._progress = progress
         self._start = start
-        self._finish = end
 
         self._es = None
         self._lf = None
@@ -44,7 +43,7 @@ class Task:
 
     def effort_numerical(self):
         if not isinstance(self._effort, int):
-            return 1
+            return Task.DEFAULT_EFFORT
         return self._effort
 
     def progress(self):
@@ -87,13 +86,10 @@ class Task:
         self._start = start
 
     def set_end(self, end):
-        self._end = end
+        self._log.debug("Setting finish value for task is deprecated, this is calculated by start and effort.")
 
     def end(self):
-        if self._finish == None:
-            return self.start() + self.__effort_offset()
-        
-        return self._finish
+        return self.start() + self.__effort_offset()
 
     def successors(self):
         return self._post
