@@ -3,7 +3,7 @@ import math
 import os
 
 import openpyxl as xl
-from openpyxl.utils import get_column_letter
+from openpyxl.utils import get_column_letter, column_index_from_string
 from openpyxl.formatting import Rule
 from openpyxl.styles import PatternFill, Font, Alignment
 from openpyxl.styles.differential import DifferentialStyle
@@ -17,6 +17,7 @@ from miroflowexport.internal.versions import common
 BG_COLOR_HORIZON = 'E2E5DE'
 BG_COLOR_PLAN = '1AA7EC'
 BG_COLOR_WEEK_HIGHLIGHT = 'E9CFEC'
+BG_COLOR_WEEKEND_HIGHLIGHT = 'D3D3D3'
 
 COLOR_PROGRESS_0_NOT_STARTED = '1AA7EC'
 COLOR_PROGRESS_1_STARTED = 'FCF4A3'
@@ -40,17 +41,31 @@ def apply_conditional_progress_coloring(ws, base_formula_plan, formula_range, co
     ws.conditional_formatting.add(formula_range, rule_plan)
 
 def apply_conditional_current_week_coloring(log, ws, color, formula_range, reference_column, reference_row):
+    next_column = get_column_letter(column_index_from_string(reference_column) + 1)
     color_fill_cell = PatternFill(bgColor = color)
     style_cell = DifferentialStyle(fill = color_fill_cell)
     rule_cell = Rule(type="expression", dxf = style_cell, stopIfTrue = False)
-    formula = "{refcol}${refrow}=WEEKNUM(TODAY(), 2)".format(
+    formula = "AND({refcol}${refrow}<=TODAY(), {nextcol}${refrow}>TODAY())".format(
             refcol = reference_column,
+            nextcol = next_column,
             refrow = reference_row,
         )
     log.debug("Applying conditional week highlight for range {} formatting using formula: {}".format(formula_range, formula))
     rule_cell.formula = [formula]
     ws.conditional_formatting.add(formula_range, rule_cell)
 
+def apply_weekend_coloring(log, ws, formula_range, color, reference_column, reference_row):
+    color_fill = PatternFill(bgColor = color)
+    style = DifferentialStyle(fill = color_fill, font = Font(color = color))
+    rule = Rule(type = "expression", dxf = style, stopIfTrue = True)
+    rule.formula = [
+        "OR(WEEKDAY({refcol}${refrow}) = 1, WEEKDAY({refcol}${refrow}) = 7)".format(
+            refcol = reference_column,
+            refrow = reference_row
+        )
+    ]
+    ws.conditional_formatting.add(formula_range, rule)
+
 def sort_by_critical_path(log, task_list):
     crit_only = [task for task in task_list if task.is_on_critical_path()]
     non_crit = [task for task in task_list if not task.is_on_critical_path()]
@@ -60,26 +75,117 @@ def sort_by_critical_path(log, task_list):
     crit_only.extend(non_crit)
     return crit_only
 
-def __tasks_to_excel_tasks(log, wb, tasks_dict, sheet_tasks, date_offset):
+def get_calendar_time_for_worktime(worktime_end, horizon_granularity):
+    if horizon_granularity == common.GRANULARITY_DAYS:
+        workdays_per_week = 5
+        full_weeks = math.floor(worktime_end / workdays_per_week)
+        remaining_days = worktime_end % workdays_per_week
+        caltime = full_weeks * 7 + remaining_days
+        if remaining_days == 0:
+            # do not count the weekend if the last day was a Friday
+            caltime -= 2
+
+        return caltime
+
+    return worktime_end
+
+def get_caltime_passed_since_offset(date_offset, horizon_granularity, date_today = datetime.date.today()):
+    if isinstance(date_offset, datetime.datetime):
+        date_offset = date_offset.date()
+
+    if horizon_granularity == common.GRANULARITY_DAYS:
+        delta = (date_today - date_offset).days
+        return delta
+
+    _, week_offset, _ = date_offset.isocalendar()
+    _, week_today, _ = date_today.isocalendar()
+    weeks_passed = week_today - week_offset
+    return weeks_passed
+
+def get_caltime_from_worktime_formula(col, row):
+    caltime_from_worktime_formula = 'ROUNDDOWN({col}{row}/5, 0) * 7 + MOD({col}{row}, 5)'.format(col = col, row = row)    
+    return caltime_from_worktime_formula
+
+def get_plan_formula(
+    horizon_granularity,
+    col_letter_task_start,
+    col_letter_task_finish,
+    col_letter_task_es,
+    col_letter_task_lf,
+    current_row,
+    caltime,
+    fill_plan = TASK_FILL_PLAN,
+    fill_horizon = TASK_FILL_HORIZON,
+):
+    if horizon_granularity == common.GRANULARITY_DAYS:
+        formula = '=IF('\
+                    "AND("\
+                        "{t_start}<={caltime},"\
+                        "{t_end}>={caltime}"\
+                    "),"\
+                    '"{fill_plan}",'\
+                    "IF("\
+                        "AND("\
+                            "{t_es}<={caltime},"\
+                            "{t_lf}>={caltime}"\
+                        "),"\
+                        '"{fill_horizon}",'\
+                        '""'\
+                    ")"\
+                ")".format(
+                    t_start = get_caltime_from_worktime_formula(col_letter_task_start, current_row),
+                    t_end = get_caltime_from_worktime_formula(col_letter_task_finish, current_row),
+                    t_es = get_caltime_from_worktime_formula(col_letter_task_es, current_row),
+                    t_lf = get_caltime_from_worktime_formula(col_letter_task_lf, current_row),
+                    caltime = caltime,
+                    fill_plan = fill_plan,
+                    fill_horizon = fill_horizon,
+                )
+        return formula
+
+    formula = '=IF('\
+                "AND("\
+                    "{col_letter_task_start}{row_num}<={caltime},"\
+                    "{col_letter_task_finish}{row_num}>={caltime}"\
+                "),"\
+                '"{fill_plan}",'\
+                "IF("\
+                    "AND("\
+                        "{col_letter_task_es}{row_num}<={caltime},"\
+                        "{col_letter_task_lf}{row_num}>={caltime}"\
+                    "),"\
+                    '"{fill_horizon}",'\
+                    '""'\
+                ")"\
+            ")".format(
+                col_letter_task_start = col_letter_task_start,
+                col_letter_task_finish = col_letter_task_finish,
+                col_letter_task_es = col_letter_task_es,
+                col_letter_task_lf = col_letter_task_lf,
+                row_num = current_row,
+                caltime = caltime,
+                fill_plan = fill_plan,
+                fill_horizon = fill_horizon,
+            )
+    return formula
+
+def __tasks_to_excel_tasks(log, wb, tasks_dict, sheet_tasks, date_offset, horizon_granularity):
     ws = wb.active
     ws.title = sheet_tasks
 
-    _, week_offset, _ = date_offset.isocalendar()
-    _, week_today, _ = datetime.date.today().isocalendar()
-    weeks_passed = week_today - week_offset
-    log.debug("Weeks passed since date offset: {}".format(weeks_passed))
+    caltime_passed = get_caltime_passed_since_offset(date_offset, horizon_granularity)
+    log.debug("Calendar time passed since date offset: {} {}".format(caltime_passed, horizon_granularity))
 
-    week_start = min([
-        task.earliest_start()
-        for task in tasks_dict.values()
-    ])
-    week_end = max([
+    worktime_start = 0
+    caltime_end = max([
         task.latest_finish()
         for task in tasks_dict.values()
     ])
-    cols_weeks = [
-        "Week {}".format(week)
-        for week in range(week_start, week_end + 1)
+    caltime_end = get_calendar_time_for_worktime(caltime_end, horizon_granularity)
+
+    cols_caltime = [
+        "{}{}".format(EXCEL_COL_NAMES[COL_TASK_WEEK_PREFIX], cal)
+        for cal in range(caltime_end + 1)
     ]
 
     header_row = [
@@ -98,7 +204,7 @@ def __tasks_to_excel_tasks(log, wb, tasks_dict, sheet_tasks, date_offset):
         ""
         for _ in header_row
     ]
-    week_num_row = [
+    caltime_row = [
         header
         for header in header_row
     ]
@@ -120,41 +226,54 @@ def __tasks_to_excel_tasks(log, wb, tasks_dict, sheet_tasks, date_offset):
         (get_column_letter(header_row.index(EXCEL_COL_NAMES[COL_TASK_ES]) + 1), get_column_letter(header_row.index(EXCEL_COL_NAMES[COL_TASK_FINISH]) + 1)),
     ]
 
-    num_cols_before_weeks = len(header_row)
-    header_row.extend(cols_weeks)
+    num_cols_before_horizon = len(header_row)
+    header_row.extend(cols_caltime)
 
-    col_letters_for_week_cols = [
-        get_column_letter(week_col)
-        for week_col in range(num_cols_before_weeks + 1, num_cols_before_weeks + 1 + len(cols_weeks))
+    col_letters_for_horizon_cols = [
+        get_column_letter(hzn_col)
+        for hzn_col in range(num_cols_before_horizon + 1, num_cols_before_horizon + 1 + len(cols_caltime))
     ]
 
     date_row_num = 2
-    formula_date_offset = "={metasheet}!{colletter}{row}".format(
+    date_offset_reference = "{metasheet}!{colletter}{row}".format(
         metasheet = excelcolumns.DEFAULT_SHEET_META,
         colletter = get_column_letter(excelcolumns.COLNUM_META_DATE_OFFSET),
         row = excelcolumns.ROWNUM_META_INFO,
     )
+    date_horizon_start = date_offset_reference
+    if horizon_granularity == common.GRANULARITY_DAYS:
+        date_horizon_start = "{start} + MOD(7 - (WEEKDAY({start}) - 2), 7)".format(
+            start = date_offset_reference,
+        )
+
+    formula_date_offset = "={ref}".format(
+        ref = date_horizon_start,
+    )
+    caltime_step = 7
+    if horizon_granularity == common.GRANULARITY_DAYS:
+        caltime_step = 1
+
     date_row.append(formula_date_offset)
     date_row.extend([
-        "={col}{row}+7".format(col = col_letter, row = date_row_num)
-        for col_letter in col_letters_for_week_cols[:-1]
+        "={col}{row}+{caltime_step}".format(col = col_letter, row = date_row_num, caltime_step = caltime_step)
+        for col_letter in col_letters_for_horizon_cols[:-1]
     ])
 
-    week_row_num = 3
-    week_num_row.extend([
+    caltime_row_num = 3
+    caltime_row.extend([
         "=WEEKNUM({col}{row})".format(col = col_letter, row = date_row_num)
-        for col_letter in col_letters_for_week_cols
+        for col_letter in col_letters_for_horizon_cols
     ])
 
     ws.append(header_row)
     ws.append(date_row)
-    for col_num in range(num_cols_before_weeks + 1, num_cols_before_weeks + 1 + len(cols_weeks)):
+    for col_num in range(num_cols_before_horizon + 1, num_cols_before_horizon + 1 + len(cols_caltime)):
         ws.cell(column = col_num, row = date_row_num).number_format = "yyyy-mm-dd"
         ws.cell(column = col_num, row = date_row_num).font = Font(size = 8)
         ws.cell(column = col_num, row = date_row_num).alignment = Alignment(text_rotation = 90)
-    ws.append(week_num_row)
-    for col_num in range(1, num_cols_before_weeks + 1 + len(cols_weeks)):
-        ws.cell(column = col_num, row = week_row_num).font = Font(b = True)
+    ws.append(caltime_row)
+    for col_num in range(1, num_cols_before_horizon + 1 + len(cols_caltime)):
+        ws.cell(column = col_num, row = caltime_row_num).font = Font(b = True)
 
     data_row_start = DATA_ROW_START
 
@@ -168,7 +287,7 @@ def __tasks_to_excel_tasks(log, wb, tasks_dict, sheet_tasks, date_offset):
     row_far_far_away_start = None
     row_far_far_away_end = None
 
-    threshold = THRESHOLD_FOLD_FUTURE_TASKS + weeks_passed
+    threshold = THRESHOLD_FOLD_FUTURE_TASKS + caltime_passed
 
     for task in sort_by_critical_path(log, list(tasks_dict.values())):
         current_row += 1
@@ -184,38 +303,25 @@ def __tasks_to_excel_tasks(log, wb, tasks_dict, sheet_tasks, date_offset):
             task.earliest_start(),
             task.latest_finish(),
             task.effort(),
-            week_start,
-            week_end,
+            worktime_start,
+            caltime_end,
             formula_task_end,
             task.progress(),
         ]
         content_row.extend([
-            '=IF('\
-                "AND("\
-                    "{col_letter_task_start}{row_num}<={week},"\
-                    "{col_letter_task_finish}{row_num}>={week}"\
-                "),"\
-                '"{fill_plan}",'\
-                "IF("\
-                    "AND("\
-                        "{col_letter_task_es}{row_num}<={week},"\
-                        "{col_letter_task_lf}{row_num}>={week}"\
-                    "),"\
-                    '"{fill_horizon}",'\
-                    '""'\
-                ")"\
-            ")".format(
+            get_plan_formula(
+                horizon_granularity = horizon_granularity,
                 col_letter_task_start = col_letter_task_start,
                 col_letter_task_finish = col_letter_task_finish,
                 col_letter_task_es = col_letter_task_es,
                 col_letter_task_lf = col_letter_task_lf,
-                row_num = current_row,
-                week = week,
+                current_row = current_row,
+                caltime = caltime,
                 fill_plan = TASK_FILL_PLAN,
                 fill_horizon = TASK_FILL_HORIZON,
             )
             # "" if week < task.earliest_start() or week > task.latest_finish() else TASK_FILL_PLAN if week >= task.start() and week <= task.end() else TASK_FILL_HORIZON
-            for week in range(week_start, week_end + 1)
+            for caltime in range(worktime_start, caltime_end + 1)
         ])
         if not wrote_progress_formula_to_log:
             log.debug("Progress formula: {}".format(content_row[-1]))
@@ -239,13 +345,13 @@ def __tasks_to_excel_tasks(log, wb, tasks_dict, sheet_tasks, date_offset):
             (row_far_far_away_start, row_far_far_away_end)
         ]
 
-    formula_base = "{}{}=".format(col_letters_for_week_cols[0], data_row_start)
+    formula_base = "{}{}=".format(col_letters_for_horizon_cols[0], data_row_start)
     log.debug("Formula base for conditional formatting: {}".format(formula_base))
 
     formula_range = "{}{}:{}{}".format(
-        col_letters_for_week_cols[0],
+        col_letters_for_horizon_cols[0],
         data_row_start,
-        col_letters_for_week_cols[-1],
+        col_letters_for_horizon_cols[-1],
         len(tasks_dict.keys()) + data_row_start
     )
     log.debug("Formula range for conditional formatting: {}".format(formula_range))
@@ -255,6 +361,25 @@ def __tasks_to_excel_tasks(log, wb, tasks_dict, sheet_tasks, date_offset):
     log.debug("Formula for horizon formatting: {}".format(formula_horizon))
     log.debug("Formula for plan formatting:    {}".format(formula_plan))
 
+    ref_column_for_week_highlight = col_letters_for_horizon_cols[0]
+    ref_row_for_week_highlight = date_row_num
+    formula_range_for_week_highlight = "{}{}:{}{}".format(
+        col_letters_for_horizon_cols[0],
+        date_row_num,
+        col_letters_for_horizon_cols[-1],
+        len(tasks_dict.keys()) + data_row_start - 1
+    )
+
+    if horizon_granularity == common.GRANULARITY_DAYS:
+        apply_weekend_coloring(
+            log = log,
+            ws = ws,
+            color = BG_COLOR_WEEKEND_HIGHLIGHT,
+            formula_range = formula_range_for_week_highlight,
+            reference_column = ref_column_for_week_highlight,
+            reference_row = ref_row_for_week_highlight,
+        )
+
     font_horizon = Font(color = BG_COLOR_HORIZON)
     
     color_fill_horizon = PatternFill(bgColor = BG_COLOR_HORIZON)
@@ -280,14 +405,6 @@ def __tasks_to_excel_tasks(log, wb, tasks_dict, sheet_tasks, date_offset):
             upper_threshold = threshold,
         )
 
-    ref_column_for_week_highlight = col_letters_for_week_cols[0]
-    ref_row_for_week_highlight = week_row_num
-    formula_range_for_week_highlight = "{}{}:{}{}".format(
-        col_letters_for_week_cols[0],
-        date_row_num,
-        col_letters_for_week_cols[-1],
-        len(tasks_dict.keys()) + data_row_start
-    )
     apply_conditional_current_week_coloring(
         log = log,
         ws = ws,
@@ -297,7 +414,7 @@ def __tasks_to_excel_tasks(log, wb, tasks_dict, sheet_tasks, date_offset):
         reference_row = ref_row_for_week_highlight,
     )
 
-    for col_letter in col_letters_for_week_cols:
+    for col_letter in col_letters_for_horizon_cols:
         ws.column_dimensions[col_letter].width = 3
 
     task_name_width = math.ceil(max([
@@ -318,13 +435,14 @@ def __tasks_to_excel_tasks(log, wb, tasks_dict, sheet_tasks, date_offset):
         log.debug("Hiding rows {} to {} ...".format(row_start, row_end))
         ws.row_dimensions.group(row_start, row_end, hidden = True)
 
-def __tasks_to_excel_meta(log, wb, sheet_meta, version, date_offset):
+def __tasks_to_excel_meta(log, wb, sheet_meta, version, date_offset, horizon_granularity):
     ws = wb.create_sheet(title = sheet_meta)
     ws.append([
         excelcolumns.COL_META_DATE_OFFSET,
         excelcolumns.COL_META_VERSION_MAJOR,
         excelcolumns.COL_META_VERSION_MINOR,
         excelcolumns.COL_META_VERSION_PATCH,
+        excelcolumns.COL_META_HORIZON_GRANULARITY,
     ])
     version_values = version.split(".")
     if not len(version_values) == 3:
@@ -335,15 +453,25 @@ def __tasks_to_excel_meta(log, wb, sheet_meta, version, date_offset):
         date_offset,
     ]
     meta_row.extend(version_values)
+    meta_row.append(str(horizon_granularity).lower())
     ws.append(meta_row)
 
 
-def tasks_to_excel(log, tasks_dict, path = "tmp/export.xlsx", sheet_tasks = "tasks", sheet_meta = "data", version = version, date_offset = datetime.date.today()):
+def tasks_to_excel(
+    log, 
+    tasks_dict, 
+    path = "tmp/export.xlsx", 
+    sheet_tasks = "tasks", 
+    sheet_meta = "data", 
+    version = version, 
+    date_offset = datetime.date.today(),
+    horizon_granularity = common.GRANULARITY_WEEKS,
+):
     log.debug("Creating Excel export for {} tasks ...".format(len(tasks_dict.values())))
     wb = xl.Workbook()
 
-    __tasks_to_excel_tasks(log, wb, tasks_dict, sheet_tasks, date_offset)
-    __tasks_to_excel_meta(log, wb, sheet_meta, version, date_offset)
+    __tasks_to_excel_tasks(log, wb, tasks_dict, sheet_tasks, date_offset, horizon_granularity)
+    __tasks_to_excel_meta(log, wb, sheet_meta, version, date_offset, horizon_granularity)
 
     log.debug("Saving Excel workbook to {} ...".format(path))
     try:
